import moment = require("moment");
import { addOvertime, delOvertime, findOvertime, findOvertimeApprove, findOvertimeByApprover, findOvertimeById, updateOvertime, updateOvertimeState } from "../data/admin/overtime";
import * as tools from "../util/tools";
import * as projectData from "../data/admin/project";
import { BizError } from "../util/bizError";
import { ERRORENUM } from "../config/errorEnum";
import { findDepartmentRole, findUserByUserIdNum } from "../data/admin/user";
import { findAskforleaveByUserIdAndType } from "../data/admin/askForLeave";
import { APPROVETYPE, ASKFORLEAVETYPE, STATED } from "../config/enum";


/**
 * 填报加班
 * @param userId 填报人
 * @param time 工时
 * @param projectId 项目号
 * @param date 填报时间
 * @param workLog 工作内容
 * @returns {isSuccess:true, message:"添加成功"}
 */
export async function addOvertimeLog(userId:number, time:number, projectId:string, date:string, workLog:string ) {
    let selectKey = moment(date).format("YYYY-MM-DD")+ ' 18:00:00'

    //查找项目编号
    let projectInfo = await projectData.findOnceProjectById(projectId);
    if ( !projectInfo ) throw new BizError(ERRORENUM.不存在该项目, `${userId} 填报 ${projectId}`);

    //获取个人信息
    let userInfo = await findUserByUserIdNum(userId);
    let approver = 0;
    if (userInfo.roleId == 6) {
        let approverInfo = await findDepartmentRole(5, userInfo.departmentId);
        approver = approverInfo.userId;
    } else {
        approver = 17;//默认给张老师审批
    }

    let id = tools.getOvertimeId(userId);

    await addOvertime(id, projectId, userId, time, selectKey, workLog, approver);

    return {isSuccess:true, message:"添加成功"};
}


/**
 * 查看
 * @param userId 填报人
 * @returns { 本周填报的工时信息, 本月工时统计 }
 */
export async function overtimeList(userId:number) {

    let overtimeLog = await findOvertime(userId);
    let projectMap = await projectData.findAllProjectMap();
    overtimeLog.sort((a, b) => {return new Date(b.dateKey).valueOf() - new Date(a.dateKey).valueOf() });
    let notApprove = 0;
    let dataList = [];
    let overtimeCount = 0;
    overtimeLog.forEach(info => {
        let {id, projectId, userId, time, dateKey, isOutside, workLog, state  } = info;
        notApprove += state == 0 ? time : 0;
        overtimeCount += time;
        let stateStr = "未审批"
        if (state) stateStr = state == APPROVETYPE.通过 ? "通过" : "未通过";
        dataList.push({
            dateKey:moment(dateKey).format("YYYY-MM-DD"),
            id, projectId, userId, time, isOutside, workLog,
            projectName:projectMap[projectId],
            state: stateStr
        });
    });
    let askforleaveCount = await findAskforleaveByUserIdAndType(userId, ASKFORLEAVETYPE.调休);
    let approveCount = await findOvertimeApprove(userId);
    
    return { dataList, notApprove, available:approveCount-askforleaveCount, overtimeCount };
}


/**
 * 修改加班
 * @param id 
 * @param userId 填报人
 * @param time 工时
 * @param projectId 项目号
 * @param date 填报时间
 * @param workLog 工作内容
 * @returns {isSuccess:true, message:"添加成功"}
 */
export async function updateOvertimeLog(id:string, userId:number, time:number, projectId:string, workLog:string ) {
    let overTimeInfo = await findOvertimeById(id);

    if (!overTimeInfo[0]) throw new BizError(ERRORENUM.目标数据已不存在)
    if (overTimeInfo[0].state == STATED.是) throw new BizError(ERRORENUM.已经通过审批的加班不能修改或删除);

    await updateOvertime(id, projectId, time, workLog);

    return {isSuccess:true, message:"添加成功"};
}


export async function deleteOvertimeLog(id:string ) {
    let overTimeInfo = await findOvertimeById(id);
    if (!overTimeInfo[0]) throw new BizError(ERRORENUM.目标数据已不存在)
    if (overTimeInfo[0].state == STATED.是) throw new BizError(ERRORENUM.已经通过审批的加班不能修改或删除);

    await delOvertime(id);

    return {isSuccess:true, message:"删除成功"};
}

export async function approveOverTime(id:string, type:number, notPassingStr:string) {
    if (type == APPROVETYPE.不通过 && !notPassingStr) {
        throw new BizError(ERRORENUM.参数错误);
    }
    if (type == APPROVETYPE.通过) notPassingStr = "";

    let overTimeInfo = await findOvertimeById(id);
    if (!overTimeInfo) throw new BizError(ERRORENUM.目标数据已不存在);
    await updateOvertimeState(id, type, notPassingStr);

    return {isSuccess:true, message:"操作成功"};
}

export async function approveOverTimeList(userId:number) {

    let overTimeList = await findOvertimeByApprover(userId);
    let dataList = [];
    let peopleMap = {};
    overTimeList.forEach(info => {
        let {userName, projectName, id, time, dateKey, workLog} = info;
        dataList.push({
            dateKey:moment(dateKey).format("YYYY-MM-DD"),
            userName, projectName, id, time, workLog
        });
        peopleMap[userName] = 1;
    });
    

    return {dataList, peopleCount:Object.keys(peopleMap).length };
}