/**
 * 工时逻辑
 * 作者:lxm
 * 包括工时填报
 */

import moment = require("moment");
import { ERRORENUM } from "../config/errorEnum";
import * as projectData from "../data/admin/project";
import * as workinglogData from "../data/admin/workinglog";
import * as overtimeData from "../data/admin/overtime";
import { BizError } from "../util/bizError";
import * as tools from "../util/tools";
import { findDepartmentMap } from "../data/admin/department";
import { selectUserByDepartmentId } from "../data/admin/user";


/**
 * 填报工时
 * @param userId 填报人
 * @param time 工时
 * @param projectId 项目号
 * @param isOutside 是否外出 true/false
 * @param date 填报时间
 * @param workLog 工作内容
 * @returns {isSuccess:true, message:"添加成功"}
 */
export async function addWorking(userId:number, time:number, projectId:string, isOutside:boolean, date:string, workLog:string ) {
    let selectKey = moment(date).format("YYYY-MM-DD")+ ' 18:00:00'
    let timeCount = await workinglogData.findWorkingLogLengthByDateKey(userId, selectKey);
    if ((timeCount + time) > 8) throw new BizError(ERRORENUM.一天只有8小时多余的时间请填加班, `${timeCount + time } >= 24`);
    // if (logCount>0) throw new BizError(ERRORENUM.重复提交, `${DateKey}工时填报重复提交`);

    //查找项目编号
    let projectInfo = await projectData.findOnceProjectById(projectId);
    if ( !projectInfo ) throw new BizError(ERRORENUM.不存在该项目, `${userId} 填报 ${projectId}`);

    let id = tools.getWorkingLogId(userId);

    let outside = isOutside ? 1 : 0;
    await workinglogData.addWorkingLog(id, projectId, userId, time, selectKey, outside, workLog);

    return {isSuccess:true, message:"添加成功"};
}


/**
 * 查看本月填报的工时  工时列表
 * 本周的规则是:周一到周日
 * @param userId 填报人
 * @returns { dataList:本周填报的工时信息, 本周工时统计 }
 */
export async function workinglogList(userId:number) {
    let weekofday = parseInt(moment().format('d')) // 计算今天是这周第几天 周日为一周中的第一天
    if (weekofday == 0) weekofday = 7;
    let s = moment().startOf('month').format('YYYY-MM-DD') + ' 00:00:00';// 周-日期
    let e = moment().format('YYYY-MM-DD') + ' 23:59:59';// 周日期

    let thisWeekLog = await workinglogData.findThisWeekWorkingLog(userId, s, e);

    thisWeekLog.sort((a, b) => {return new Date(b.dateKey).valueOf() - new Date(a.dateKey).valueOf() });

    let timeCount = 0;
    let distinctMap = {};
    thisWeekLog.forEach(info => {
        info.dateKey = moment(info.dateKey).format("YYYY-MM-DD");
        timeCount += info.time;
        info.isOutside = info.isOutside == 1;
        distinctMap[info.dateKey] = 1;
    });
    
    return { dataList:thisWeekLog, timeCount, dateCount:Object.keys(distinctMap).length };
}


/**
 * 修改已填报的工时信息
 * @param userId 填报人
 * @param id 属性的数据类型
 * @param time 工时
 * @param projectId 项目号
 * @param isOutside 是否外出 true/false
 * @param workLog 工作内容
 * @returns { isSuccess:true, message:"修改成功" }
 */
export async function updateWorkinglog(userId:number, id:string, time:number, projectId:string, isOutside:boolean, workLog:string) {
    let s = moment().startOf('month').format('YYYY-MM-DD') + ' 00:00:00';// 周-日期
    let e = moment().format('YYYY-MM-DD') + ' 23:59:59';// 周日期
    let thisWeekLog = await workinglogData.findThisWeekWorkingLog(userId, s, e);

    let checkDateKey = 0;
    thisWeekLog.forEach(info => {
        if (info.id == id) {
            checkDateKey = new Date(info.dateKey).getDate();
        }
    });
    if (!checkDateKey) throw new BizError(ERRORENUM.未找到该数据, id);

    let count = time;
    thisWeekLog.forEach(info => {
        if (new Date(info.dateKey).getDate() == checkDateKey && info.id != id ) {
            count += info.time;
        }
    });
    if (count > 8) throw new BizError(ERRORENUM.一天只有8小时多余的时间请填加班, `${ count } > 8`);

    let outside = isOutside ? 1 : 0;
    await workinglogData.updateWorkinglog(id, projectId, time, outside, workLog);

    return { isSuccess:true, message:"修改成功" };
}


/**
 * 根据传入id删除填报工时
 * @param userId 填报人
 * @param id 属性的数据类型
 * @returns { isSuccess:true, message:"删除成功" }
 */
export async function delWorkinglog(userId:number, id:string,) {

    let dataInfo = await workinglogData.findWorkingLogById(id);
    if (!dataInfo) throw new BizError(ERRORENUM.未找到该数据, id);

    await workinglogData.deleteWorkingLogById(id);

    return { isSuccess:true, message:"删除成功" };
}


/**
 * 工时管理列表 按时间分
 * @param page 页数
 * @param startMs 开始时间
 * @param endMs 结束时间
 * @returns {dataList:工时管理列表, total:工时列表统计 }
 */
export async function adminWorkLogForTime(page:number, startMs:string, endMs:string) {
    let s = null;
    let e = null;
    if (startMs && endMs) {
        s = moment(startMs).format("YYYY-MM-DD") + ' 00:00:00';
        e = moment(endMs).format("YYYY-MM-DD") + ' 23:59:59';
    }

    let {dataList, count} = await workinglogData.findAllWorkingLogToPage((page-1 )*10, s, e);

    const DepartmentMap = await findDepartmentMap();

    let result = [];

    let distinctMap = {};
    dataList.forEach(info => {
        let year = moment(info.dateKey).year();
        let month = moment(info.dateKey).month()+1;
        let day = moment(info.dateKey).date();
        result.push({
            year,
            month,
            day,
            name:info.name,
            departmentName : DepartmentMap[info.departmentId],
            projectName:info.projectName,
            projectId:info.projectId,
            workLog:info.workLog,
            time: info.time
        });
        distinctMap[day] = 1;
    });

    return {dataList:result, total:count, dateCount:Object.keys(distinctMap).length };
}


/**
 * 工时管理列表 按项目分
 * @param page 页数
 * @param startMs 开始时间
 * @param endMs 结束时间
 * @returns {dataList:工时管理列表, total:工时列表统计 };
 */
export async function adminWorkLogForProject(page:number, startMs:string, endMs:string) {
    let s = null;
    let e = null;
    if (startMs && endMs) {
        s = moment(startMs).format("YYYY-MM-DD") + ' 00:00:00';
        e = moment(endMs).format("YYYY-MM-DD") + ' 23:59:59';
    }
    
    let workingLog = await workinglogData.findAllWorkingLogToProject(s, e);

    let distinctMap = {};// => {name:{id:"", count:0}}

    workingLog.forEach(info => {
        if (!distinctMap[info.projectName]) distinctMap[info.projectName] = {id:info.projectId, count:0 }
        distinctMap[info.projectName].count += info.time;
    });

    let result = [];
    for (let name in distinctMap) {
        let {id, count} = distinctMap[name];
        let dayCount = Math.ceil(count/8 * 100)/100
        result.push({
            name,
            id,
            hostCount:count,
            dayCount
        });
    }

    return {dataList:result.splice((page-1)*10, 10 ), total:result.length };
}


/**
 * 导出按时间分的工时管理列表
 * @param startMs 开始时间
 * @param endMs 结束时间
 * @returns {dataList:工时管理列表, name:`【按时间分工时】.xlsx`}
 */
export async function outPutAdminWorkingLogListForTime(startMs:string, endMs:string) {
    let s = null;
    let e = null;
    if (startMs && endMs) {
        s = moment(startMs).format("YYYY-MM-DD") + ' 00:00:00';
        e = moment(endMs).format("YYYY-MM-DD") + ' 23:59:59';
    }
    
    let workingLog = await workinglogData.findAllWorkingLog(s, e);

    /**加班 */
    let overtimeList = await overtimeData.findOvertimeByTime(s, e);
    let overtimeMap = {};
    overtimeList.forEach(info => {
        let {dateKey, name, time} = info;
        let dateKeyStr = moment(dateKey).format("YYYY-MM-DD");
        if (!overtimeMap[dateKeyStr]) overtimeMap[dateKeyStr] = {};
        if (!overtimeMap[dateKeyStr][name]) overtimeMap[dateKeyStr][name] = time;
    });

    const DepartmentMap = await findDepartmentMap();

    let result = [
        ["日期年","日期月","日期日","部门","姓名","项目名称","项目编号","时长/小时","工作内容", "加班时长"]
    ];

    workingLog.forEach(info => {
        let year = moment(info.dateKey).year();
        let month = moment(info.dateKey).month()+1;
        let day = moment(info.dateKey).date();
        let departmentName = DepartmentMap[info.departmentId];
        let dateKeyStr = moment(info.dateKey).format("YYYY-MM-DD");
        let overtimeCount = "";
        if (overtimeMap[dateKeyStr] && overtimeMap[dateKeyStr][info.name]) overtimeCount = ""+overtimeMap[dateKeyStr][info.name];
        let subList = [
            year,
            month,
            day,
            departmentName,
            info.name,
            info.projectName,
            info.projectId,
            info.time,
            info.workLog,
            overtimeCount
        ]
        result.push(subList);
    });


    return {dataList:result, name:`【按时间分工时】.xlsx`};
}


/**
 * 导出按项目分的工时管理列表
 * @param startMs 开始时间
 * @param endMs 结束时间
 * @returns {dataList:工时管理列表, name:`【按项目分工时】.xlsx`}
 */
export async function outPutAdminWorkingLogListForProject(startMs:string, endMs:string) {
    let s = null;
    let e = null;
    if (startMs && endMs) {
        s = moment(startMs).format("YYYY-MM-DD") + ' 00:00:00';
        e = moment(endMs).format("YYYY-MM-DD") + ' 23:59:59';
    }
    let workingLog = await workinglogData.findAllWorkingLogToProject(s, e);

    let overtimeList = await overtimeData.findOvertimeByTime(s, e);
    let overtimeMap = {};
    overtimeList.forEach(info => {
        let {dateKey, name, time, projectName} = info;
        if (!overtimeMap[projectName]) overtimeMap[projectName] = 0;
        overtimeMap[projectName] += time;
    });

    let distinctMap = {};// => {name:{id:"", count:0}}

    workingLog.forEach(info => {
        if (!distinctMap[info.projectName]) distinctMap[info.projectName] = {id:info.projectId, count:0 }
        distinctMap[info.projectName].count += info.time;
    });

    let result = [
        ["项目名称","项目编号","时长/小时","时长/人天", "加班时长/小时"]
    ];
    for (let name in distinctMap) {
        let {id, count} = distinctMap[name];
        let dayCount = Math.ceil(count/8 * 100)/100;
        let overtimeCount = overtimeMap[name] || 0;
        result.push([name, id, count, dayCount, overtimeCount]);
    }
    
    return {dataList:result, name:`【按项目分工时】.xlsx`};
}


/**
 * 部门主管看的工时列表
 */
export async function departmentWorklog(departmentId:number, startTime:string, endTime:string, userId:number, page:number, projectId:string) {
    let selectParam:any = {departmentId};
    if (userId) selectParam.userId = userId;
    if (startTime) selectParam.startTime = moment( startTime).format("YYYY-MM-DD")+" 00:00:00";
    if (endTime) selectParam.endTime = moment( endTime).format("YYYY-MM-DD")+" 00:00:00";
    if (projectId) selectParam.projectId = projectId;

    let dbList = await workinglogData.findWorkingLogByTimeOrUserId(departmentId, selectParam, (page-1)*10);
    let count = await workinglogData.findWorkingLogCountByTimeOrUserId(departmentId, selectParam);
    let dataList = [];
    dbList.forEach(info => {
        dataList.push({
            dateKey:moment(info.dateKey).format("YYYY-MM-DD"),
            name:info.name,
            time:info.time,
            projectName : info.projectName,
            isOutside:info.isOutside,
            workLog:info.workLog
        });
    }); 

    return {dataList, count}
}

export async function getDepartmentUserList(departmentId:number) {
    let userList = await selectUserByDepartmentId(departmentId);
    let dataList = [];
    userList.forEach(info => {
        dataList.push({
            value:info.userId,
            label:info.name
        });
    });
    return {dataList};
}